/* demo.c */

#include "lp_lib.h"

int demo()
{
    lprec *lp;
    int Ncol, *colno = NULL, j, ret = 0;
    REAL *row = NULL;

    /* We will build the model row by row, so we start with creating a model with 0 rows and 4 columns */

    Ncol = 4; /* there are four variables in the model */
    lp = make_lp(0, Ncol);
    if (lp == NULL)
        ret = 1; /* couldn't construct a new model... */

    if (ret == 0)
    {
        /* optional naming of variables */
        set_col_name(lp, 1, "dP1+");
        set_col_name(lp, 2, "dP1-");
        set_col_name(lp, 3, "dP2+");
        set_col_name(lp, 4, "dP2-");

        /* create space large enough for one row */
        colno = (int *)malloc(Ncol * sizeof(*colno));
        row = (REAL *)malloc(Ncol * sizeof(*row));
        if ((colno == NULL) || (row == NULL))
            ret = 2;
    }

    if (ret == 0)
    {
        set_add_rowmode(lp, TRUE); /* makes building the model faster if it is done rows by row */

        /* construct first row (balance constraint): x1 - x2 + x3 - x4*/
        j = 0;

        colno[j] = 1; /* first column */
        row[j++] = 1;

        colno[j] = 2; /* second column */
        row[j++] = -1;

        colno[j] = 3; /* third column */
        row[j++] = 1;

        colno[j] = 4; /* fourth column */
        row[j++] = -1;

        /* add the row to lpsolve */
        if (!add_constraintex(lp, j, row, colno, EQ, 0))
            ret = 3;
    }

    if (ret == 0)
    {
        /* construct second row (transmission constraint line 1-2):  
        0.333x1 - 0.333x2 - 0.333x3 + 0.333x4 < 100 - 60;
        */
        j = 0;

        colno[j] = 1; /* first column */
        row[j++] = 0.333;

        colno[j] = 2; /* second column */
        row[j++] = -0.333;

        colno[j] = 3; /* third column */
        row[j++] = -0.333;

        colno[j] = 4; /* fourth column */
        row[j++] = 0.333;

        /* add the row to lpsolve */
        if (!add_constraintex(lp, j, row, colno, LE, 40))
            ret = 3;
    }

    if (ret == 0)
    {
        /* construct third row (transmission constraint line 2-3):  
        0.333x1 -0.333x2 + 0.667x3 - 0.667x4 < 100 - 70;
        */
        j = 0;

        colno[j] = 1; /* first column */
        row[j++] = 0.333;

        colno[j] = 2; /* second column */
        row[j++] = -0.333;

        colno[j] = 3; /* third column */
        row[j++] = 0.667;

        colno[j] = 4; /* fourth column */
        row[j++] = -0.667;

        /* add the row to lpsolve */
        if (!add_constraintex(lp, j, row, colno, LE, 30))
            ret = 3;
    }

    if (ret == 0)
    {
        /* construct fourth row (transmission constraint line 1-3):  
        0.667x1 - 0.667x2 + 0.333x3 - 0.333x4 < 100 - 110;
        */
        j = 0;

        colno[j] = 1; /* first column */
        row[j++] = 0.667;

        colno[j] = 2; /* second column */
        row[j++] = -0.667;

        colno[j] = 3; /* third column */
        row[j++] = 0.333;

        colno[j] = 4; /* fourth column */
        row[j++] = -0.333;

        /* add the row to lpsolve */
        if (!add_constraintex(lp, j, row, colno, LE, -10))
            ret = 3;
    }

    if (ret == 0)
    {
        set_add_rowmode(lp, FALSE); /* rowmode should be turned off again when done building the model */

        /* set the objective function (20 x1 +  20 x2 +  30 x3 +  30 x4) */
        j = 0;

        colno[j] = 1; /* first column */
        row[j++] = 20;

        colno[j] = 2; /* second column */
        row[j++] = 20;

        colno[j] = 3; /* first column */
        row[j++] = 30;

        colno[j] = 4; /* second column */
        row[j++] = 30;

        /* set the objective in lpsolve */
        if (!set_obj_fnex(lp, j, row, colno))
            ret = 4;
    }

    if (ret == 0)
    {
        /* set the object direction to minimize */
        set_minim(lp);

        /* print model to screen and write to file  */
        write_LP(lp, stdout);
        write_lp(lp, "model.lp");

        /* Only warnings and error messages will be shown */
        set_verbose(lp, IMPORTANT);

        /* Solve the problem */
        ret = solve(lp);
        if (ret == OPTIMAL)
            ret = 0;
        else
            ret = 5;
    }

    if (ret == 0)
    {
        /* get the results */

        /* objective value */
        printf("Objective value: %f\n", get_objective(lp));

        /* variable values */
        get_variables(lp, row);
        for (j = 0; j < Ncol; j++)
            printf("%s: %f\n", get_col_name(lp, j + 1), row[j]);

        /* we are done now */
    }

    /* free allocated memory */
    if (row != NULL)
        free(row);
    if (colno != NULL)
        free(colno);

    if (lp != NULL)
    {
        /* free up all memory used by lpsolve */
        delete_lp(lp);
    }

    return (ret);
}

int main()
{
    demo();
}